<?php

namespace Drupal\wordsonline_connector\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\wordsonline_connector\WordsOnlineConst;
use Drupal\tmgmt\Entity\Job;
use Drupal\tmgmt\JobInterface;
use Drupal\wordsonline_connector\Entity\WOFile;
use Drupal\wordsonline_connector\Common\ZipHandle;
use Drupal\wordsonline_connector\WordsOnlineState;
use Drupal\wordsonline_connector\WordsOnlineStatus;
use Symfony\Component\DependencyInjection\ContainerInterface;
use GuzzleHttp\ClientInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Archiver\ArchiverManager;
use Drupal\Core\File\FileSystemInterface;
use Drupal\tmgmt_file\Format\FormatManager;
use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\wordsonline_connector\Plugin\tmgmt\Translator\WordsOnlineTranslator;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
//use Drupal\Core\File\FileUrlGenerator;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\tmgmt\Entity\JobItem;
use Drupal\node\Entity\Node;

/**
 * Controller for handle order.
 */
class OrderController extends ControllerBase {
  use StringTranslationTrait;

  /**
   * Guzzle HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $client;

  /**
   * Database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * ModuleExtensionList.
   *
   * @var Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleExtensionList;

  /**
   * Messenger.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * Archiver.
   *
   * @var Drupal\Core\Archiver\ArchiverManager
   */
  protected $archiver;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The file format manager.
   *
   * @var \Drupal\tmgmt_file\Format\FormatManager
   */
  protected $formatManager;

  /**
   * Constructor.
   *
   * @param \GuzzleHttp\ClientInterface $client
   *   Http client.
   * @param \Drupal\Core\Database\Connection $database
   *   Database connection.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   Messenger.
   * @param \Drupal\Core\Archiver\ArchiverManager $archiver
   *   Archiver manager.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   File System.
   * @param \Drupal\tmgmt_file\Format\FormatManager $formatManager
   *   The file format manager.
   * @param \Drupal\Core\Extension\ModuleExtensionList
   *   The list module extension list.
   */
  public function __construct(ClientInterface $client, Connection $database, Messenger $messenger, ArchiverManager $archiver, FileSystemInterface $fileSystem, FormatManager $formatManager, ModuleExtensionList $moduleExtensionList) {
    $this->client = $client;
    $this->database = $database;
    $this->messenger = $messenger;
    $this->archiver = $archiver;
    $this->fileSystem = $fileSystem;
    $this->formatManager = $formatManager;
    //$this->jobEntityQuery = $jobEntityQuery;
    //$this->fileUrlGenerator = $fileUrlGenerator;
    $this->moduleExtensionList = $moduleExtensionList;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $client = $container->get('http_client');
    $database = $container->get('database');
    $messenger = $container->get('messenger');
    $archiver = $container->get('plugin.manager.archiver');
    $fileSystem = $container->get('file_system');
    $manager = $container->get('plugin.manager.tmgmt_file.format');
    // $jobEntityQuery = $container->get('entity_type.manager')
    //   ->getStorage("tmgmt_job")->getQuery('AND');
    //$fileUrlGenerator = $container->get('file_url_generator');
    $moduleExtensionList = $container->get('extension.list.module');
    return new static(
        $client,
        $database,
        $messenger,
        $archiver,
        $fileSystem,
        $manager,
        //$jobEntityQuery,
       // $fileUrlGenerator,
        $moduleExtensionList
    );
  }

  /**
   * Open confirm form.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The request from client.
   */
  public function openMyModal(Request $request) {
    $request_guid = $request->query->get("request_guid");
    $job_id = $request->query->get("job_id");
    $allowSubmmit = "true";
    if ($request->query->get("allowSubmmit")) {
      $allowSubmmit = $request->query->get("allowSubmmit");
    }
    $modal_form = $this->formBuilder()->getForm(
            '\Drupal\wordsonline_connector\Form\OrderConfirmForm',
            $allowSubmmit,
            $job_id,
            $request_guid
    );
    $render["form"] = $modal_form;
    return $render;
  }

  // public function makeOutDateTranslated($job_item_id) {
  //   $job_item = JobItem::load($job_item_id);
  //   if($job_item) {
  //     $job = $job_item->getJob();
  //     $continuous = $this->database
  //     ->query("SELECT tjid FROM tmgmt_job where job_type ='continuous' and translator =:translator and source_language=:source_language and target_language=:target_language",
  //     [
  //       ':source_language' => $job->getSourceLangCode(),
  //       ':target_language' => $job->getTargetLangcode(),
  //       ':translator' => WordsOnlineConst::MODULE_ID
  //     ]
  //     )
  //     ->fetchAll();
  //     if($continuous !=null && count($continuous)>0){
  //       $node = Node::load($job_item->getItemId());
  //       $translation = $node->getTranslation($job->getTargetLangcode());
  //        // Set the outdated flag to true.
  //       $translation->content_translation_outdated->value = 1;
  //       $translation->save();
  //     }
  //   }
  // }

  /**
   * Check quote status.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The request from client.
   *
   * @return Symfony\Component\HttpFoundation\JsonResponse
   *   A json response.
   */
  public function checkQuoted(Request $request) {
    $has_change = FALSE;
    $key = NULL;
    if ($request->query->get("key")) {
      $key = $request->query->get("key");
    }
    $top = "10";
    $skip = 0;
    if ($request->query->get("page")) {
      $skip = $request->query->get("page");
    }
    if ($request->query->get("top")) {
      $top = $request->query->get("top");
    }
    $list = $this->getDataList($key, $skip, $top);
    if ($list != NULL) {
      $data = $list->list;
      $count = $list->count;
      if ($count > 0) {
        foreach ($data as $row) {
          $results = $this->getWordsOnlineJobRecordByRequestGuid($row->requestGuid);
          if ($results != NULL) {
            $job_id = $results["job_id"];
            $status = $results["status"];
            $job = Job::load($job_id);
            if ($row->state == WordsOnlineState::AUTOMATION_FAILED
              && $status != WordsOnlineConst::JOB_AUTOMATION_FAILED
            ) {
              $this->updateWordsOnlineJobStatus($job_id, WordsOnlineConst::JOB_AUTOMATION_FAILED);
              $has_change = TRUE;
              continue;
            }
            if (($row->status == WordsOnlineStatus::UNPAID
              || $row->status == WordsOnlineStatus::QUOTE_SUBMITTED)
              && $status != WordsOnlineConst::JOB_QUOTED
            ) {
              $this->updateWordsOnlineJobStatus($job_id, WordsOnlineConst::JOB_QUOTED);
              $has_change = TRUE;
              continue;
            }
            if($row->status == WordsOnlineStatus::IN_PROGRESS_STATUS && $status != WordsOnlineConst::JOB_QUOTED){
              $this->updateWordsOnlineJobStatus($job_id, WordsOnlineConst::JOB_QUOTED);
              $has_change = TRUE;
              continue;
            }
            if($row->status == WordsOnlineStatus::PREPARING_QUOTE){
              $has_change = TRUE;
              continue;
            }
            if ($row->status == WordsOnlineStatus::DELIVERED
              || $row->status == WordsOnlineStatus::IMPORT_FAIL_STATUS
              || $row->status == WordsOnlineStatus::IMPORT_FAILED_STATUS
            ) {
              if ($status != WordsOnlineConst::JOB_DELIVERED
                && $status != WordsOnlineConst::JOB_IMPORTED
                && $status != WordsOnlineConst::JOB_FINISHED
              ) {
                $has_change = TRUE;
                continue;
              }
            }
            if ($job) {
              $state = $job->getState();
              if ($state == WordsOnlineConst::STATE_FINISHED
                && $row->status != WordsOnlineStatus::FINISHED_STATUS
              ) {
                $has_change = TRUE;
                continue;
              }
            }
          }
        }
      }
    }
    return new JsonResponse(
      [
        "status" => "success",
        "changed" => $has_change
      ]
    );
  }

  /**
   * Get WordsOnline Job record by request guid.
   * 
   * @param string $request_guid
   * Request guid.
   */
  public function getWordsOnlineJobRecordByRequestGuid($request_guid) {
    $results = $this->database
    ->query(
        "SELECT request_guid,job_id,status 
        FROM wordsonline_connector_jobs 
        WHERE request_guid =:request_guid  LIMIT 1 ",
        [":request_guid" => (string) $request_guid]
    )
    ->fetchAssoc();
    return $results;
  }

  /**
   * Update status of WordsOnline Job table
   * 
   * @param int $job_id
   * Job id.
   * @param string $status
   * Status.
   */
  public function updateWordsOnlineJobStatus($job_id, $status) {
    $this->database
    ->update(WordsOnlineConst::JOB_TABLE)
    ->condition("job_id", $job_id)
    ->fields(
      [
       "status" =>  $status,
      ]
    )
    ->execute();
  }

  /**
   * Call request to WordsOnline api for action.
   * 
   * @param array $auth
   * Form param.
   * @param string $request_guid
   * Request guid.
   * @param string $status
   * Status.
   */
  public function requestAction($auth,$request_guid,$status){
    $response = wordsonline_connector_get_token("POST", $auth);
    $response = json_decode($response, TRUE);
    $token = $response["access_token"];
    wordsonline_connector_request_action(
      $request_guid,
      $status,
      "",
      $token
    );
  }

  /**
   * Get WordsOnline job by job id.
   * 
   * @param int $job_id
   * Job id.
   */
  public function getWordsOnlineConnectorJobRecordByJobId($job_id){
    $f_ret = $this->database
    ->query(WordsOnlineConst::GET_WORDSONLINE_JOB_BY_JOB_ID_QUERY,
      [
        ':job_id' => $job_id
      ]
    )
    ->fetchAssoc();
    return $f_ret;
  }

  /**
   * Display list wordsonline job.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The request from client.
   */
  public function jobList(Request $request) {
    $key = NULL;
    if ($request->query->get("key")) {
      $key = $request->query->get("key");
    }
    $top = "10";
    $skip = 0;
    if ($request->query->get("page")) {
      $skip = $request->query->get("page");
    }
    if ($request->query->get("top")) {
      $top = $request->query->get("top");
    }

    $job_filter_id = "0";
    if ($request->query->get("job_id")) {
      $job_filter_id = $request->query->get("job_id");
    }
    $header = [
      "id" => $this->t("Id"),
      "label" => $this->t("Label"),
      "project_name" => $this->t("Project Name"),
      "request_id" => $this->t("Request Id"),
      "request_name" => $this->t("Request Name"),
      "lang_name" => $this->t("Language"),
      "due_date" => $this->t("Due Date"),
      "status" => $this->t("Status"),
      "opt" => $this->t("Operations"),
    ];
    $refresh_link = base_path().'admin/tmgmt/wordsonline/jobs?top='.$top ."&page=".$skip ;
    if($key != NULL){
      $refresh_link = $refresh_link ."&key=".$key;
    }
    if($job_filter_id != "0"){
      $refresh_link = $refresh_link ."&job_id=".$job_filter_id;
    }
    $form['grant_type'] = [
      '#type' => 'hidden',
      '#default_value' => $refresh_link,
      '#value' => $refresh_link,
      "#attributes" => [
        "id" => "wo_request_url",
      ],
    ];
    $form["#prefix"] =
        "<div> <a id='btnRefresh'  href='#' 
         class='wol-btn btn-layout'>Refresh Page</a>
         </div> <div id='wol-loader'></div>";
    $list = $this->getDataList($key, $skip, $top, $job_filter_id);
    $rows = [];
    $count = 0;
    $pageCount = 0;
    if ($list != NULL) {
      $data = $list->list;
      $count = $list->count;
      $numOfPage = is_numeric($top) ? (int) $top : 10;
      if ($count % $numOfPage > 0) {
        $pageCount = $count / $numOfPage + 1;
      }
      else {
        $pageCount = $count / $numOfPage;
      }
      $configs = $this->getConfig();
      $prj_guid ='';
      if ($configs != NULL) {
        $auth["username"] = $configs["username"];
        $auth["password"] = $configs["password"];
        $auth["scope"] = $configs["scope"];
        $auth["grant_type"] = $configs["grant_type"];
        $prj_guid = $configs["project_key"];
      }
      $results = $this->database
      ->query(WordsOnlineConst::ALL_JOB_QUERY)
      ->fetchAll();
      foreach ($data as $row) {
        $job_id = "";
        $operation = "";
        $due_date = "";
        $label = "";
        $status = WordsOnlineConst::NOT_AVAILABLE;
        $sourceLink = "";
        foreach ($results as $res) {
          if ($res->request_guid == $row->requestGuid) {
            $job_id = $res->job_id;
            $job = Job::load($job_id);
            if ($job) {
              $label = html_entity_decode($job->label());
              $status = $row->status;
              $due_date = $job->getSetting("due_date");
              $state = $job->getState();
              $jobItems = array_values($job->getItems());
              $sourceLink = $this->getSourceLink($jobItems);
              if ($row->status == WordsOnlineStatus::IMPORTED_STATUS && $res->status == WordsOnlineConst::JOB_DELIVERED) {
                $this->updateWordsOnlineJobStatus($job_id, WordsOnlineConst::JOB_IMPORTED);
                $res->status = WordsOnlineConst::JOB_IMPORTED;
              }
              if ($state == WordsOnlineConst::STATE_FINISHED) {
                if ($row->status != WordsOnlineStatus::FINISHED_STATUS) {
                  $this->requestAction($auth, $row->requestGuid, WordsOnlineStatus::FINISHED_STATUS);
                  $status = WordsOnlineStatus::FINISHED_STATUS;
                }
                $download_link = $this->getDownloadLink($job_id,$row->requestGuid);
                $view_job_link = $this->getViewJobLink($jobItems,$job_id);
                $operation = $this->t( $view_job_link . $download_link . $sourceLink );
                $this->updateWordsOnlineJobStatus($job_id, WordsOnlineConst::JOB_FINISHED);
                break;
              }
              //hotfix old bug 
              if ($state != WordsOnlineConst::STATE_FINISHED && $row->status == WordsOnlineStatus::FINISHED_STATUS) {
                $this->requestAction($auth, $row->requestGuid, WordsOnlineStatus::IMPORTED_STATUS);
                $status = WordsOnlineStatus::IMPORTED_STATUS;
                $edit = $this->getReviewLink($jobItems, $job_id);
                $operation =$this->t( $this->getViewXlfLink($job_id,$res->request_guid,'Reimport',$edit) . $sourceLink);
                $this->updateWordsOnlineJobStatus($job_id, WordsOnlineConst::IMPORTED_STATUS);
                break;
              }
              //end hotfix
              if ($row->state == WordsOnlineState::AUTOMATION_FAILED) {
                $this->updateWordsOnlineJobStatus($job_id, WordsOnlineConst::JOB_AUTOMATION_FAILED);
                $status = WordsOnlineConst::JOB_AUTOMATION_FAILED;
                break;
              }
              if ($row->status == WordsOnlineStatus::IN_PROGRESS_STATUS) {
                $operation =$this->t( $this->getConfirmLink($job_id, $res->request_guid,"&allowSubmmit=false") . $sourceLink);
                $this->updateWordsOnlineJobStatus($job_id, WordsOnlineConst::JOB_QUOTED);
                break;
              }
              if ($row->state == WordsOnlineState::QUOTE_SUBMITTED
                && ($row->status == WordsOnlineStatus::UNPAID
                || $row->status == WordsOnlineStatus::QUOTE_SUBMITTED)
                ) {
                $operation =$this->t( $this->getConfirmLink($job_id, $res->request_guid) . $sourceLink);
                $this->updateWordsOnlineJobStatus($job_id, WordsOnlineConst::JOB_QUOTED);
                break;
              }
              if ($row->state == WordsOnlineState::PAYMENT_FAILED
                  && ($row->status == WordsOnlineStatus::UNPAID
                  || $row->status == WordsOnlineStatus::QUOTE_SUBMITTED)
                  ) {
                $operation =$this->t( $this->getConfirmLink($job_id, $res->request_guid) . $sourceLink);
                break;
              }
              if ($res->status == WordsOnlineConst::JOB_IMPORTED) {
                if ($row->status != WordsOnlineStatus::IMPORTED_STATUS) {
                  $this->requestAction($auth, $row->requestGuid, WordsOnlineStatus::IMPORTED_STATUS);
                  $status = WordsOnlineStatus::IMPORTED_STATUS;
                }
                $edit = $this->getReviewLink($jobItems, $job_id);
                $operation =$this->t( $this->getViewXlfLink($job_id,$res->request_guid,'Reimport',$edit) . $sourceLink);
                break;
              }
              if ($row->status == WordsOnlineStatus::DELIVERED
                || $row->status == WordsOnlineStatus::IMPORT_FAIL_STATUS
                || $row->status == WordsOnlineStatus::IMPORT_FAILED_STATUS
                ) {
                if ($res->status != WordsOnlineConst::JOB_DELIVERED) {
                  $this->updateWordsOnlineJobStatus($job_id, WordsOnlineConst::JOB_DELIVERED);
                }
                $link_text = "Import";
                if ($row->status == WordsOnlineStatus::IMPORT_FAIL_STATUS
                || $row->status == WordsOnlineStatus::IMPORT_FAILED_STATUS) {
                  $link_text = "Reimport";
                }
                $operation =$this->t( $this->getViewXlfLink($job_id,$res->request_guid,$link_text) . $sourceLink);
              }
              break;
            }
          }
        }
        $rows[] = [
          "id" => $job_id,
          "label" => $label,
          "project_name" => $row->projectId,
          "request_id" => $row->requestId,
          "request_name" => $row->requestName,
          "lang_name" =>
          $row->sourceLanguageName . " > " . $row->targetLanguageName,
          "due_date" => $due_date,
          "status" => $status,
          "opt" => $operation,
        ];
      }
    }
    $form["searh_prefix"] = [
      "#markup" => '<div class="wol-search-field">',
    ];
    $form["searh_data"] = [
      "#type" => "textfield",
      "#name" => "job_data",
      "#value" => $key,
      "#placeholder" =>  $this->t(WordsOnlineConst::SEARCH_PLACEHOLDER),
    ];
    $form["actions"] = [
      "#type" => "button",
      "#value" => $this->t("Search"),
      "#name" => "job-search",
      "#attributes" => [
        "class" => ["btn", "btn-layout", "ml-3", "wol-btn"],
        "onclick" => "searchJob()",
      ],
    ];
    $pageLimits = [];
    $pageLimits["10"] = "10";
    $pageLimits["15"] = "15";
    $pageLimits["20"] = "20";
    $pageLimits["25"] = "25";
    $form["page_limit"] = [
      "#type" => "select",
      "#required" => TRUE,
      "#name" => "job_page",
      "#default_value" => $top,
      "#options" => $pageLimits,
      "#attributes" => ["onchange" => "changePageLimit()"],
      '#value' => $top,
    ];
    $job_filters=[];
    $job_ids = $this->database
    ->query(
      WordsOnlineConst::ALL_JOB_FOR_FILTER_QUERY,
        [
          ':prj_guid' => $prj_guid
        ]
      )
    ->fetchAll();
    $job_filters["0"] = $this->t("Select Job");
    foreach($job_ids as $tjid){
      $val ="Job (" . $tjid->tjid . ")";
      $job_filters[(string)$tjid->tjid] = $val;
    }
    $form["job_filter"] = [
      "#type" => "select",
      "#name" => "job_filter",
      "#default_value" => $job_filter_id,
      "#options" => $job_filters,
      "#attributes" => ["onchange" => "changePageLimit()"],
      '#value' =>$job_filter_id,
    ];
    $form["searh_surfix"] = [
      "#markup" => "</div>",
    ];
    $form["table"] = [
      "#type" => "table",
      "#header" => $header,
      "#rows" => $rows,
      "#empty" => $this->t("No records found"),
      "#attributes" => ["class" => ["wol-table"]],
    ];
    $form["pager"] = [
      "#type" => "pager",
    ];
    $pagging = $this->generatePagingHtml($pageCount, $skip);
    $form["info_prefix"] = [
      "#markup" => $pagging,
    ];
    $form["#attached"]["library"][] = "wordsonline_connector/wordsonline_job";
    return $form;
  }

  /**
   * Generate paging html
   */
  public function generatePagingHtml($pageCount, $skip){
    $pagging = "<div class='pagination'>";
    if ($pageCount > 0) {
      $pagging = $pagging . "<a href='#' class='wo-page' data-page='1'>&laquo;</a>";
      if ($skip == "" || $skip == NULL) {
        $skip = 1;
      }
      for ($i = 1; $i <= $pageCount; $i++) {
        $active = "";
        if ($i == $skip) {
          $active = "active";
        }
        $pagging =
          $pagging .
          "<a class='$active wo-page' href='#' data-page='$i'>" .
          $i .
          "</a>";
      }
      $pagging =
        $pagging .
        "<a href='#' class='wo-page' data-page='$pageCount'>&raquo;</a></div>";
    }
    $pagging = $pagging . "</div>";
    return $pagging;
  }

  /**
   * Get view job link.
   * 
   * @param array $jobItems
   * Job items.
   * @param int $job_id
   * Job id.
   */
  public function getViewJobLink($jobItems, $job_id){
    $url ='';
    if (count($jobItems) > 1) {
      $url = base_path() .
        "admin/tmgmt/jobs/" .
        $job_id;
    }
    else {
      $url = base_path() .
        "admin/tmgmt/items/" .
        $jobItems[0]->id() .
        "?destination=" .
        base_path() .
        "admin/tmgmt/jobs/" .
        $job_id;
    }
    return " <a href='$url' target='_blank'   class='btn'  title='View Job'>View Job</a> ";
  }

  /**
   * Get download link.
   * 
   * @param int $job_id
   * Job id.
   * @param string $request_guid
   * Request guid.
   */
  public function getDownloadLink($job_id,$request_guid){
    $down = base_path() .
    WordsOnlineConst::VIEW_XLF_LINK .
    $job_id.'&request_guid='.$request_guid;
  return $this->t($add.
    " <a href=':url' class='use-ajax btn'  
    data-dialog-type='modal' 
    title='download'>Download</a> ",
    [
      ":url" => $down
    ]
  );
  }

  /**
   * Get confirm link.
   * 
   * @param int $job_id
   * Job id.
   * @param string $request_guid
   * Request guid.
   * @param string $add
   * Add more param.
   */
  public function getConfirmLink($job_id, $request_guid,$add=''){
    $link = base_path() .
    WordsOnlineConst::CONFIRM_ORDER_LINK .
    $job_id.'&request_guid='.$request_guid.$add;
    return $this->t(
    " <a href=':url' class='use-ajax btn'  
     data-dialog-type='modal' title='View Quote'
     >View Quote</a> ",
    [
      ":url" => $link,
    ]
    );
  }

  /**
   * Get review link.
   * 
   * @param array $jobItems
   * Job items.
   * @param int $job_id
   * Job id.
   */
  public function getReviewLink($jobItems, $job_id){
    $link = '';
    if (count($jobItems) > 1) {
      $url = base_path() .
        "admin/tmgmt/jobs/" .
        $job_id;
        $link =" <div class='wo-dropdown'><button class='dropbtn'>Review</button><div class='wo-dropdown-content'>";
        foreach($jobItems as $ji){
          $job_item_id = $ji->id();
          $link .="<a href='".base_path() .
          "admin/tmgmt/items/" .
          $job_item_id .
          "?destination=" .
          base_path() .
          "admin/tmgmt/jobs/" .
          $job_id."' hreflang='und'  target='_blank'>".$ji->getSourceLabel()."(Job Item " . $job_item_id . ")</a>";
        }
        $link .= "</div></div> ";
    }
    else {
        foreach($jobItems as $ji){
          $job_item_id = $ji->id();
          $url = base_path() .
          "admin/tmgmt/items/" .
          $job_item_id .
          "?destination=" .
          base_path() .
          "admin/tmgmt/jobs/" .
          $job_id;
          $link = " <a href='" . $url . "' target='_blank' 
          class='btn' title='Review'>Review</a> ";
          break;
        }
    }
    return  $link;
  }

  /**
   * Get source link.
   * 
   * @param array $jobItems
   * Job items.
   */
  public function getSourceLink($jobItems){
    $link =" <div class='wo-dropdown'><button class='dropbtn'>Source</button><div class='wo-dropdown-content'>";
    foreach($jobItems as $ji){
      $item_id = $ji->getItemId();
      $sourceUrl =  $ji->getSourceUrl()->toString();
      if($sourceUrl == "FALSE" || $sourceUrl == "1"){
        $sourceUrl = "/node/".$item_id;
      }
      if (substr($sourceUrl , 0, 1 ) === '/') {
        $sourceUrl = substr($sourceUrl, 1);
      }
      $sourceUrl = base_path() . $sourceUrl;
      $sourceLabel =  $ji->getSourceLabel();
      $link .="<a href='" . $sourceUrl . "' hreflang='und' target='_blank'>" . $sourceLabel ."(Job Item " . $ji->id() . ")</a>";
    }
    $link .= "</div></div> ";
    return  $link;
  }

  /**
   * Get view xlf link.
   * 
   * @param int $job_id
   * Job id.
   * @param string $request_guid
   * Request guid.
   * @param string $link_text
   * Link text.
   * @param string $add
   * Add more string.
   */
  public function getViewXlfLink($job_id,$request_guid,$link_text ='Import',$add =''){
    $down = base_path() .
    WordsOnlineConst::VIEW_XLF_LINK .
    $job_id.'&request_guid='.$request_guid;
  return $this->t($add.
    " <a href=':url' class='use-ajax btn'  
    data-dialog-type='modal' 
    title='download'>Download</a>  <a  class='btn' onclick='reimport(@job_id,@request_guid
    )'  title='@link_text'>@link_text </a> ",
    [
      ":url" => $down,
      "@job_id" => $job_id,
      "@link_text" => $link_text,
      '@request_guid' =>  "'" . (string) $request_guid . "'",
    ]
  );
  }

  /**
   * Get list requests.
   *
   * @param string $filter
   *   The filter string.
   * @param int $skip
   *   Skip record.
   * @param string $top
   *   Number of record per page.
   * @param string $job_filter_id
   * Job id.
   */
  public function getDataList($filter, $skip, $top = "10", string $job_filter_id = "0") {
    $page = 0;
    if ($skip != NULL && $skip != "") {
      $page = (int) $skip;
      if ($page > 0) {
        $page = $page - 1;
      }
    }
    if($job_filter_id != NULL &&  $job_filter_id !="0"){
     $record = $this->getWordsOnlineConnectorJobRecordByJobId((int)$job_filter_id);
     if($record != NULL){
      $filter = $record["request_name"];
     }
    }
   
    $page = $page * 10;
    $token = $this->getToken();
    if ($token != NULL) {
      $url = WordsOnlineConst::API_URL .
        'Requests?$skip=' .
        $page .
        '&$top=' .
        $top .
        '&$orderby=CreatedAt desc ';
      if ($filter != NULL && $filter != "") {
        // $searchId = "or contains(tolower(ProjectId),tolower(%27" .
        //   $filter .
        //   "%27)) ";
        $searchId = "";
        if (is_numeric($filter)) {
          $searchId = " or RequestId eq  " . $filter . " ";
        }
        // $url = $url .
        //   '&$filter=(contains(tolower(TargetLanguage),tolower(%27' .
        //   $filter .
        //   "%27)) or contains(tolower(SourceLanguage),tolower(%27" .
        //   $filter .
        //   "%27)) or contains(tolower(RequestName),tolower(%27" .
        //   $filter .
        //   "%27))  " .
        //   $searchId .
        //   ")";
        $url = $url .
          '&$filter=(contains(tolower(RequestName),tolower(%27' .
          $filter .
          "%27))  " .
          $searchId .
          ")";
      }
     
      $result = $this->client->get(
        $url,
        [
          "headers" => [
            "Authorization" => "Bearer " . $token,
            "Accept" => "application/json",
            "Referer" => "ClientAPI",
          ],
          "timeout" => 3600,
        ]
      );
      if ($result->getStatusCode() == 200) {
        $data = json_decode($result->getBody()->getContents());
        if ($data->status == 1) {
          return $data->result;
        }
      }
    }
    return NULL;
  }

  /**
   * Get quote info for job.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The request from client.
   *
   * @return Symfony\Component\HttpFoundation\JsonResponse
   *   A json response.
   */
  public function getQuote(Request $request) {
    $res["status"] = "error";
    if ($request->query->get("job_id")) {
      $job_id = $request->query->get("job_id");
      $request_guid = $request->query->get("request_guid");
      $token = $this->getToken();
      if ($token != NULL) {
        if ($request_guid != NULL) {
          $quotes = wordsonline_connector_get_quote($request_guid, $token);
          if ($quotes != NULL && $quotes->status == 1) {
            $res["status"] = "success";
            $res["data"] = $quotes->result;
          }
        }
        return new JsonResponse($res);
      }
      else {
        return new JsonResponse($res);
      }
    }
    else {
      return new JsonResponse($res);
    }
  }

  /**
   * Get the token.
   */
  public function getToken(){
    $results = $this->database->query(WordsOnlineConst::SELECT_CONFIG)->fetchAssoc();
    $token = NULL;
    if ($results != NULL) {
      $auth["username"] = $results["username"];
      $auth["password"] = $results["password"];
      $auth["scope"] = $results["scope"];
      $auth["grant_type"] = $results["grant_type"];
      $response = wordsonline_connector_get_token('POST', $auth);
      $response = json_decode($response, TRUE);
      $token = $response['access_token'];
    }
    else {
      $this->messenger->addError(
        $this->pleaseContactHelpDeskMessage()
      );
    }
    return $token;
  }

  /**
   * Get records ordered or delivered
   * 
   * @param string $table_name
   * The name of table.
   */
  public function getRecordsForCheckHasFile($table_name){
    $records = $this->database->select($table_name, "wol");
    $records->fields("wol");
    $records->condition(
      "wol.status",
      [
        WordsOnlineConst::JOB_ORDERED,
        WordsOnlineConst::JOB_DELIVERED,
      ],
      "in"
    );
    $records->orderBy("wol.job_id", "DESC");
    $res = $records->execute()->fetchAll();
    return $res;
  }

  /**
   * Check has delivery files.
   *
   * @return Symfony\Component\HttpFoundation\JsonResponse
   *   A json response.
   */
  public function checkHasFiles() {
    $token = $this->getToken();
    if ($token != NULL) {
      $records = $this->getRecordsForCheckHasFile(WordsOnlineConst::JOB_TABLE); 
      $has_change = FALSE;
      foreach ($records as $row) {
        $files = wordsonline_connector_get_request_files_list($row->request_guid, $token);
        if ($files != NULL) {
          if ($files->result != NULL && count($files->result) > 0) {
            $job = Job::load($row->job_id);
            if ($job) {
              $auto_import = $job->getSetting("auto_import");
              $translator = $job->getTranslator();
              $translator->setSetting("xliff_processing", TRUE);
              $translator->save();
              $is_auto = $translator->isAutoAccept();
              if ($row->status == WordsOnlineConst::JOB_ORDERED) {
                $has_change = TRUE;
              }
              if ($auto_import == TRUE
                || $auto_import == 1
                || $is_auto == TRUE
                || $is_auto == 1
              ) {
                $is_import = FALSE;
                $this->updateWordsOnlineJobStatus($row->job_id, WordsOnlineConst::JOB_DELIVERED);
                foreach ($files->result as $fi) {
                  $file_guid = $fi->guid;
                  $data = wordsonline_connector_download_file(
                    $row->request_guid,
                    $file_guid,
                    $token
                  );
                  $fName = "public://" . $fi->name;
                  file_put_contents($fName, $data);
                  $zip = new ZipHandle($fName, $this->archiver, $this->fileSystem);
                  $fileContent = $zip->fileContent;
                  $this->fileSystem->delete(
                    $fName
                  );
                  $is_has_error = FALSE;
                  if ($fileContent != NULL
                    && count($fileContent) > 0
                  ) {
                    foreach ($fileContent as $fcontent) {
                      // if (!$this->importTranslation(
                      //   $job,
                      //   $fcontent
                      // )) {
                      //   $this->messenger->addError("an error occured when import job {$row->job_id} with zip file {$fName}");
                      //   $this->sendErrorToWordsOnline(
                      //     $row->request_guid,
                      //     $fcontent,
                      //     $token
                      //   );
                      //   $is_has_error = TRUE;
                      // }
                      $msg = $this->importTranslation(
                        $job,
                        $fcontent
                      );

                      if($msg != "") {
                        $this->messenger->addError("an error occured when import job {$row->job_id} with zip file {$fName}");
                        wordsonline_connector_request_action(
                          $row->request_guid,
                          WordsOnlineStatus::IMPORT_FAIL_STATUS,
                          $msg,
                          $token
                        );
                        $is_has_error = TRUE;
                      }
                    }
                  }
                  if ($is_has_error == TRUE) {
                    continue;
                  }
                  // if ($is_auto == TRUE || $is_auto == 1) {
                  //   wordsonline_connector_request_action(
                  //       $row->request_guid,
                  //       WordsOnlineStatus::FINISHED_STATUS,
                  //       "",
                  //       $token
                  //     );
                  //   $this->updateWordsOnlineJobStatus($row->job_id, WordsOnlineConst::JOB_FINISHED);
                  // }
                  // else {
                    wordsonline_connector_request_action(
                        $row->request_guid,
                        WordsOnlineStatus::IMPORTED_STATUS,
                        "",
                        $token
                      );
                    $this->updateWordsOnlineJobStatus($row->job_id, WordsOnlineConst::JOB_IMPORTED);
                  //}
                  $is_import = TRUE;
                }
                if ($is_import) {
                  $has_change = TRUE;
                }
              }
              else {
                $this->updateWordsOnlineJobStatus($row->job_id, WordsOnlineConst::JOB_DELIVERED);
              }
            }
          }
        }
      }
      return new JsonResponse(
        [
          "status" => "success",
          "changed" => $has_change,
        ]
      );
    }
    else {
      return new JsonResponse(["status" => "error", "changed" => FALSE]);
    }
  }

  /**
   * Imports the given translation data.
   *
   * @param \Drupal\tmgmt\JobInterface $job
   *   The job to import translations for.
   * @param string $translation
   *   The translation data.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   *   Throws an exception if XLF plugin does not exist.
   * @throws \Drupal\tmgmt\TMGMTException
   *   Throws an exception in case of a neeror.
   */
  public function importTranslation(JobInterface $job, $translation) {
    // $pattern = "/<bpt[^>]*>/";
    // $translation = preg_replace($pattern, "", $translation);
    // $pattern = "/<\/bpt[^>]*>/";
    // $translation = preg_replace($pattern, "", $translation);
    // $pattern = "/<ept[^>]*>/";
    // $translation = preg_replace($pattern, "", $translation);
    // $pattern = "/<\/ept[^>]*>/";
    // $translation = preg_replace($pattern, "", $translation);
    // $pattern = "/<ph [^>]*>/";
    // if (preg_match_all($pattern, $translation, $matches)) {
    //   foreach ($matches as $val) {
    //     $target = str_replace("<ph", "&lt;img", $val);
    //     $target = str_replace(">", "&gt;", $target);
    //     $translation = str_replace($val, $target, $translation);
    //   }
    // }
    // $pattern = "/<x [^>]*>/";
    // if (preg_match_all($pattern, $translation, $matches)) {
    //   foreach ($matches as $val) {
    //     $translation = str_replace($val, '&lt;br /&gt;', $translation);
    //   }
    // }
    // $pattern = "/<note>(.*?)<\/note>/";
    // if (preg_match($pattern, $translation, $matches)) {
    //   foreach ($matches as $val) {
    //     $valSource = "<note>" . $val . "</note>";
    //     $valTarget = "";
    //     $translation = str_replace($valSource, $valTarget, $translation);
    //   }
    // }
    //libxml_use_internal_errors(TRUE);
    $xliff = $this->formatManager->createInstance("xlf");
    // if (!$xliff->validateImport($translation, FALSE)) {
    //   libxml_use_internal_errors(FALSE);
    //   return FALSE;
    // }
    $messages = $this->getXliffError($job, $translation);
    if ( $messages != NULL && !empty($messages) && $messages != ""){
     // libxml_use_internal_errors(FALSE);
      return $messages;
    }
    if ($data = $xliff->import($translation, FALSE)) {
      $job->addTranslatedData(
        $data,
        NULL,
        TMGMT_DATA_ITEM_STATE_TRANSLATED
      );
      $job->addMessage("The translation has been received.");
    }
    // else {
    //   libxml_use_internal_errors(FALSE);
    //   return $messages;
    // }
    // libxml_use_internal_errors(FALSE);
    return "";
  }

  /**
   * Get Error of xliff xml data.
   * 
   * @param \Drupal\tmgmt\JobInterface $job
   * Job.
   * @param string $importedXML
   * Xliff content.
   */
  public function getXliffError($job, $importedXML){
    $xml = simplexml_load_string($importedXML);
    $xml->registerXPathNamespace('xliff', 'urn:oasis:names:tc:xliff:document:1.2');
    $phase = $xml->xpath("//xliff:phase[@phase-name='extraction']");
    if ($phase) {
     $phase = reset($phase);
    }
    else {
     return 'The imported file is missing required XLIFF phase information.';
    }
    // Check if the job has a valid job reference.
    if (!isset($phase['job-id'])) {
     return 'The imported file does not contain a job reference.';
    }
    // Attempt to load the job if none passed.
    $job = (Job::load((int) $phase['job-id']));
    if (empty($job)) {
     return "The imported file job id " . $phase['job-id'] . " is not available.";
    }
    // Compare source language.
    if (!isset($xml->file['source-language']) || $job->getRemoteSourceLanguage() != $xml->file['source-language']) {
     $file_language = empty($xml->file['source-language']) ? t('none') : $xml->file['source-language'];
     $messages = 'The imported file source language ' . $file_language . ' does not match the job source language ' . $job->getRemoteSourceLanguage() . '.';
     return $messages;
    }
    // Compare target language.
    if (!isset($xml->file['target-language']) || $job->getRemoteTargetLanguage() != $xml->file['target-language']) {
     $file_language = empty($xml->file['target-language']) ? t('none') : $xml->file['target-language'];
     $messages = 'The imported file target language ' . $file_language . ' does not match the job target language ' . $job->getRemoteTargetLanguage() . '.';
     return $messages;
    }
    $targets = $this->getImportedTargets($xml, $job);

    if (empty($targets)|| $targets == NULL) {
      return 'The imported file seems to be missing translation.';
    }
    // In case we do not do xliff processing we cannot do the elements
    // count validation.
    // if (!$job->getSetting('xliff_processing')) {
    //   return "";
    // }

    // $reader = new \XMLReader();
    // $xliff_validation = $job->getSetting('xliff_validation');

    // foreach ($targets as $id => $target) {
    //   $array_key = \Drupal::service('tmgmt.data')->ensureArrayKey($id);
    //   $job_item = JobItem::load(array_shift($array_key));
    //   $count = 0;
    //   $reader->XML('<translation>' . $target['#text'] . '</translation>');
    //   while ($reader->read()) {
    //     if (in_array($reader->name, array('translation', '#text'))) {
    //       continue;
    //     }
    //     $count++;
    //   }

    //   if (!isset($xliff_validation[$id]) || $xliff_validation[$id] != $count) {
    //     $key = \Drupal::service('tmgmt.data')->ensureStringKey($array_key);
    //     return 'Failed to validate semantic integrity of ' . $key . ' element. Please check also the HTML code of the element in the review process.';
    //   }
    // }
    return "";
  }

  /**
   * Get imported targets.
   * 
   * @param SimpleXMLElement $importedXML
   * Xml element.
   * @param \Drupal\tmgmt\JobInterface $job
   * Job.
   * 
   */
  public function getImportedTargets($importedXML,$job){
    $importedTransUnits = NULL;
    $reader = new \XMLReader();
    foreach ($importedXML->xpath('//xliff:trans-unit') as $unit) {
      if (!$job->getSetting('xliff_processing')) {
        $importedTransUnits[(string) $unit['id']]['#text'] = (string) $unit->target;
        continue;
      }

      $reader->XML($unit->target->asXML());
      $reader->read();
      $importedTransUnits[(string) $unit['id']]['#text'] =
        $this->processForImport($reader->readInnerXML(), $job);
    }
    return  $importedTransUnits;
  }

  /**
   * Processes trans-unit/target to rebuild back the HTML.
   *
   * @param string $translation
   *   Job data array.
   * @param \Drupal\tmgmt\JobInterface $job
   *   Translation job.
   *
   * @return string
   */
  public function processForImport($translation, JobInterface $job) {
    // In case we do not want to do xliff processing return the translation as
    // is.
    if (!$job->getSetting('xliff_processing')) {
      return $translation;
    }

    $reader = new \XMLReader();
    $reader->XML('<translation>' . $translation . '</translation>');
    $text = '';

    while ($reader->read()) {
      // If the current element is text append it to the result text.
      if ($reader->name == '#text' || $reader->name == '#cdata-section') {
        $text .= $reader->value;
      }
      elseif ($reader->name == 'x') {
        if ($reader->getAttribute('ctype') == 'lb') {
          $text .= '<br />';
        }
      }
      elseif ($reader->name == 'ph') {
        if ($reader->getAttribute('ctype') == 'image') {
          $text .= '<img';
          while ($reader->moveToNextAttribute()) {
            // @todo - we have to use x-html: prefixes for attributes.
            if ($reader->name != 'ctype' && $reader->name != 'id') {
              $text .= " {$reader->name}=\"{$reader->value}\"";
            }
          }
          $text .= ' />';
        }
      }
    }
    return $text;
  }

  /**
   * Set request fail and send mail.
   *
   * @param string $request_guid
   *   Guid of request.
   * @param string $xmlData
   *   Xml content.
   * @param string $token
   *   The token for authorization.
   */
  public function sendErrorToWordsOnline($request_guid, $xmlData, $token) {
    $messages = wordsonline_connector_get_error_message($xmlData);
    wordsonline_connector_request_action(
      $request_guid,
      WordsOnlineStatus::IMPORT_FAIL_STATUS,
      $messages,
      $token
    );
  }

  /**
   * Import file.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The request from client.
   *
   * @return Symfony\Component\HttpFoundation\JsonResponse
   *   A json response.
   */
  public function reimport(Request $request) {
    if ($request->query->get("job_id")) {
      $job_id = $request->query->get("job_id");
      $request_guid = $request->query->get("request_guid");
      $ret = $this->database
        ->query(WordsOnlineConst::SELECT_CONFIG)
        ->fetchAssoc();
      if ($ret != NULL) {
        $auth["username"] = $ret["username"];
        $auth["password"] = $ret["password"];
        $auth["scope"] = $ret["scope"];
        $auth["grant_type"] = $ret["grant_type"];
        $response = wordsonline_connector_get_token("POST", $auth);
        $response = json_decode($response, TRUE);
        $token = $response["access_token"];
          $files = wordsonline_connector_get_request_files_list(
            $request_guid,
            $token
          );
          if ($files != NULL) {
            if ($files->result != NULL
              && count($files->result) > 0
            ) {
              $status = "downloaded";
              $job = Job::load($job_id);
              if ($job) {
                $is_import = FALSE;
                $translator = $job->getTranslator();
                $translator->setSetting(
                  "xliff_processing",
                  TRUE
                );
                $translator->save();
                $is_auto = $translator->isAutoAccept();
                foreach ($files->result as $fi) {
                  $file_guid = $fi->guid;
                  $data = wordsonline_connector_download_file(
                    $request_guid,//$request["request_guid"],
                    $file_guid,
                    $token
                  );
                  $fName = "public://" . $fi->name;
                  file_put_contents($fName, $data);
                  $zip = new ZipHandle($fName, $this->archiver, $this->fileSystem);
                  $fileContent = $zip->fileContent;
                  $this->fileSystem->delete(
                    $fName
                  );
                  $is_has_error = FALSE;
                  if ($fileContent != NULL
                    && count($fileContent) > 0
                  ) {
                    foreach ($fileContent as $fcontent) {
                      // if (!$this->importTranslation(
                      //   $job,
                      //   $fcontent
                      // )) {
                      //   $this->messenger->addError("an error occured when import job {$job_id} with zip file {$fName}");
                      //   $this->sendErrorToWordsOnline(
                      //     $request["request_guid"],
                      //     $fcontent,
                      //     $token
                      //   );
                      //   $is_has_error = TRUE;
                      // }
                      $msg = $this->importTranslation(
                        $job,
                        $fcontent
                      );
                      if($msg !="") {
                        $this->messenger->addError("an error occured when import job {$job_id} with zip file {$fName}");
                        wordsonline_connector_request_action(
                          $request_guid,//$request["request_guid"],
                          WordsOnlineStatus::IMPORT_FAIL_STATUS,
                          $msg,
                          $token
                        );
                        $is_has_error = TRUE;
                      }
                    }
                  }
                  if ($is_has_error == TRUE) {
                    continue;
                  }
                  // if ($is_auto == TRUE || $is_auto == 1) {
                  //   wordsonline_connector_request_action(
                  //     $request_guid,
                  //       WordsOnlineStatus::FINISHED_STATUS,
                  //       "",
                  //       $token
                  //     );
                    
                  //     $this->updateWordsOnlineJobStatus($job_id, WordsOnlineConst::JOB_FINISHED);
                  // }
                  // else {
                    wordsonline_connector_request_action(
                      $request_guid,
                        WordsOnlineStatus::IMPORTED_STATUS,
                        "",
                        $token
                      );
                     
                      $this->updateWordsOnlineJobStatus($job_id, WordsOnlineConst::JOB_IMPORTED);
                   
                  //}
                  $is_import = TRUE;
                }
                if ($is_import) {
                  $status = "success";
                }
              }
              return new JsonResponse(["status" => $status]);
            }
          }
       
      }
      else {
        $this->messenger->addError(
          $this->pleaseContactHelpDeskMessage()
        );
      }
    }
    return new JsonResponse(["status" => "error"]);
  }

  /**
   * Get config from wordsonline table.
   */
  public function getConfig() {
    $ret = $this->database
    ->query(WordsOnlineConst::SELECT_CONFIG)
    ->fetchAssoc();
    return $ret;
  }

  /**
   * Display xlf file for review.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The request from client.
   */
  public function viewXlf(Request $request) {
    $xmlData = [];
    $fName ="";
    if ($request->query->get("job_id")) {
      $job_id = $request->query->get("job_id");
      $request_guid = $request->query->get("request_guid");
      $token = $this->getToken();
      if ($token != NULL) {
        if ($request_guid != NULL) {
          $files = wordsonline_connector_get_request_files_list(
            $request_guid,
            $token
          );
          if ($files != NULL) {
            foreach ($files->result as $fi) {
              $file_guid = $fi->guid;
              $data = wordsonline_connector_download_file(
                $request_guid,
                $file_guid,
                $token
              );
              $fName = "public://" . $fi->name;
              file_put_contents($fName, $data);
              $zip = new ZipHandle($fName, $this->archiver, $this->fileSystem);
              $fileContent = $zip->fileContent;
              $fileNames = $zip->fileNames;
              //$this->fileSystem->delete($fName);
              if ($fileContent != NULL
                && count($fileContent) > 0
              ) {
                $i = 0;
                foreach ($fileContent as $fcontent) {
                  array_push($xmlData, new WOFile(
                    $fileNames[$i],
                    $fcontent
                  ));
                  $i++;
                }
              }
            }
          }
        }
      }
     
    }
    $modal_form = $this->formBuilder()->getForm(
      '\Drupal\wordsonline_connector\Form\ViewXlfForm',
      $xmlData,
      $fName
    );
    $render["form"] = $modal_form;
    return $render;
  }

  /**
   * Get error message when can't call wordsonline api.
   * 
   */
  public function pleaseContactHelpDeskMessage(){
    return  $this->t(
      'Cannot find WordsOnline provider settings. 
      Please contact <a href="mailto:@url">@url<a/> 
      to obtain WordsOnline provider settings.',
      [
        "@url" => WordsOnlineConst::HELPDESK_MAIL,
      ]
      );
  }

  /**
   * Get xlf file content.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The request from client.
   */
  public function getFileContent(Request $request) {
    $result = "";
    if ($request->query->get("fileName")) {
      $fileName = $request->query->get("fileName");
      $tempPath = "temporary://wordsonline_connector/$fileName";
      if (!file_exists($tempPath)) {
        return new JsonResponse(
          [
            "status" => "error",
            "result" => "",
            "messages" => "File $fileName is not exists",
          ]
        );
      }
      $result = file_get_contents($tempPath);
    }
    else {
      return new JsonResponse(
        [
          "status" => "error",
          "result" => "",
          "messages" => "Paramerter is incorrect",
        ]
      );
    }
    return new JsonResponse(
      [
        "status" => "success",
        "result" => $result,
        "messages" => "",
      ]
    );
  }

  /**
   * Get zip file content.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The request from client.
   */
  public function downloadJobFile(Request $request) {
    $fileName = $request->query->get("fileName");
    $uri = 'public://' . $fileName;
    $headers = [
      'Content-Type'     => 'application/zip',
      'Content-Disposition' => 'attachment;filename="' . $fileName . '"',
    ];
    return new BinaryFileResponse($uri, 200, $headers, TRUE);
  }

  /**
   * Display list wordsonline support language.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The request from client.
   */
  public function supportLanguages(Request $request) {
    $header = [
      "code" => $this->t("Drupal Language Code"),
      "lang_name" => $this->t("Language Name"),
      "wol_code" => $this->t("WordsOnline Language Code")
    ];
    $rows = [];
    $form["#prefix"] =
        "<div><a  href='".base_path() . WordsOnlineConst::IMPORT_SUPPORT_LANG_URL."' 
         class='wol-btn btn-layout'>Re-import support languages</a>
         </div>";
    $results = $this->database
      ->query(
        WordsOnlineConst::GET_ALL_WORDSONLINE_SUPPORT_LANGUAGES_QUERY
      )
      ->fetchAll();
    foreach ($results as $res) {
      $rows[] = [
        "code" => $res->code,
        "lang_name" => $res->lang_name,
        "wol_code" => $res->wol_code
      ];

    }
    $form["table"] = [
      "#type" => "table",
      "#header" => $header,
      "#rows" => $rows,
      "#empty" => $this->t("No records found"),
      "#attributes" => ["class" => ["wol-table"]],
    ];
    $form["#attached"]["library"][] = "wordsonline_connector/wordsonline_support_language";
    return $form;
  }

  /**
   * reImport list wordsonline support language.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The request from client.
   */
  public function importSupportLanguages(Request $request) {
     try{
      $npath = $this->moduleExtensionList->getPath(WordsOnlineConst::MODULE_ID);
      $import = $this->fileSystem->realpath($npath . '/js/languages.json');//$this->fileUrlGenerator->generateAbsoluteString($npath . '/js/languages.json');
      $content = file_get_contents($import);
      $langs = json_decode($content);
      foreach ($langs as $lang) {
        $results = $this->database
        ->query(
          WordsOnlineConst::GET_WORDSONLINE_SUPPORT_LANGUAGE_BY_CODE_QUERY,
            [":code" => (string) $lang->code]
        )
        ->fetchAssoc();
        if ($results != NULL) {
          $this->database
          ->update(WordsOnlineConst::LANGUAGE_JOB_TABLE)
          ->condition("code", $lang->code)
          ->fields(
            [
              'code' => $lang->code,
              'lang_name' => $lang->lang_name,
              'wol_code' => $lang->wol_code
            ]
          )
          ->execute();
        } else {
          $this->database->insert(WordsOnlineConst::LANGUAGE_JOB_TABLE)
          ->fields(['code', 'lang_name', 'wol_code'])
          ->values([
            'code' => $lang->code,
            'lang_name' => $lang->lang_name,
            'wol_code' => $lang->wol_code
          ])
          ->execute();
        }
       
      }
      $this->messenger->addMessage('Re-import supported languages successfully');
     }catch(\Exception $e) {
      $this->messenger->addError($e->getMessage());
     }
     $target_url = base_path() . WordsOnlineConst::SUPPORT_LANG_URL;
     $response = new RedirectResponse($target_url);
     $response->send();
  }

}
